using System;
using static System.Console;
using static System.Math;

delegate void Notify();

// A model class for a game of Tic Tac Toe.
class Board {
    int[,] square = new int[3, 3];    // 0 = empty, 1 = player 1, 2 = player 2
    
    public int player = 1;     // whose turn it is
    public int winner = 0;     // who has won
    public int win_x, win_y, win_dx, win_dy;    // vector indicating winning squares
    
    public event Notify changed;    // fires whenever the board changes
    
    public int this[int x, int y] {
        get => square[x, y];
    }
    
    // Return true if the current player has three in a row,
    // starting at (x, y) and moving in direction (dx, dy).
    bool all(int x, int y, int dx, int dy) {
        for (int i = 0 ; i < 3 ; ++i) 
            if (square[x + i * dx, y + i * dy] != player)
                return false;
        
        win_x = x; win_y = y; win_dx = dx; win_dy = dy;
        return true;
    }

    // Return true if the current player has won.
    bool checkWin() {
        // check rows and columns
        for (int i = 0 ; i < 3 ; ++i)
            if (all(i, 0, 0, 1) || all(0, i, 1, 0))
                return true;
                
        // check diagonals
        return all(0, 0, 1, 1) || all(2, 0, -1, 1);
    }
    
    // Make the current player play at (x, y).    Return true if the move was legal.
    public bool move(int x, int y) {
        if (square[x, y] > 0)    // square is occupied
            return false;
            
        square[x, y] = player;
        if (checkWin())
            winner = player;
        else player = 3 - player;
        
        changed();
        return true;
    }
}

// A view to play Tic Tac Toe in a terminal window.
class TextView {
    Board board = new Board();
    
    TextView() {
        draw();
        board.changed += draw;
    }
    
    static int posToNum(int x, int y) => y * 3 + x + 1;
    
    static void numToPos(int num, out int x, out int y) {
        y = (num - 1) / 3;
        x = (num - 1) % 3;
    }
    
    void draw() {
        Clear();
        
        for (int y = 0 ; y < 3 ; ++y) {
            for (int x = 0 ; x < 3 ; ++x) {
                switch (board[x, y]) {
                    case 0: Write(posToNum(x, y)); break;
                    case 1: Write('X'); break;
                    case 2: Write('O'); break;
                }
                Write(' ');
            }
         WriteLine();
         WriteLine();
        }
    }
    
    // controller
    void run() {
        do {
            int x, y;
            do {
                numToPos(Read() - '0', out x, out y);
            } while (!board.move(x, y));
        } while (board.winner == 0);
    }
    
    static void Main() {
        new TextView().run();
    }
}
